@@ -1,7 +1,7 @@
 #!/usr/local/bin/python2.7
 # -*- coding: utf-8 -*-
 ###############################################################################
-# $Id: gdal_edit.py 37709 2017-03-14 14:52:15Z rouault $
+# $Id: gdal_edit.py c1fbba933ab495ea744145c45921b91a2053c513 2018-04-17 18:56:03 +1000 Ben Elliston $
 #
 #  Project:  GDAL samples
 #  Purpose:  Edit in place various information of an existing GDAL dataset
@@ -34,10 +34,12 @@ import sys
 from osgeo import gdal
 from osgeo import osr
 
+
 def Usage():
     print('Usage: gdal_edit [--help-general] [-ro] [-a_srs srs_def] [-a_ullr ulx uly lrx lry]')
     print('                 [-tr xres yres] [-unsetgt] [-a_nodata value] [-unsetnodata]')
     print('                 [-offset value] [-scale value]')
+    print('                 [-colorinterp_X red|green|blue|alpha|gray|undefined]*')
     print('                 [-unsetstats] [-stats] [-approx_stats]')
     print('                 [-gcp pixel line easting northing [elevation]]*')
     print('                 [-unsetmd] [-oo NAME=VALUE]* [-mo "META-TAG=VALUE"]*  datasetname')
@@ -50,15 +52,16 @@ def ArgIsNumeric(s):
     i = 0
 
     while i < len(s):
-        if ( s[i] < '0' or s[i] > '9') and s[i] != '.' and s[i] != 'e' and s[i] != '+' and s[i] != '-':
+        if (s[i] < '0' or s[i] > '9') and s[i] != '.' and s[i] != 'e' and s[i] != '+' and s[i] != '-':
             return False
         i = i + 1
 
     return True
 
+
 def gdal_edit(argv):
 
-    argv = gdal.GeneralCmdLineProcessor( argv )
+    argv = gdal.GeneralCmdLineProcessor(argv)
     if argv is None:
         return -1
 
@@ -83,58 +86,59 @@ def gdal_edit(argv):
     open_options = []
     offset = None
     scale = None
+    colorinterp = {}
 
     i = 1
     argc = len(argv)
     while i < argc:
         if argv[i] == '-ro':
             ro = True
-        elif argv[i] == '-a_srs' and i < len(argv)-1:
-            srs = argv[i+1]
+        elif argv[i] == '-a_srs' and i < len(argv) - 1:
+            srs = argv[i + 1]
             i = i + 1
-        elif argv[i] == '-a_ullr' and i < len(argv)-4:
-            ulx = float(argv[i+1])
+        elif argv[i] == '-a_ullr' and i < len(argv) - 4:
+            ulx = float(argv[i + 1])
             i = i + 1
-            uly = float(argv[i+1])
+            uly = float(argv[i + 1])
             i = i + 1
-            lrx = float(argv[i+1])
+            lrx = float(argv[i + 1])
             i = i + 1
-            lry = float(argv[i+1])
+            lry = float(argv[i + 1])
             i = i + 1
-        elif argv[i] == '-tr' and i < len(argv)-2:
-            xres = float(argv[i+1])
+        elif argv[i] == '-tr' and i < len(argv) - 2:
+            xres = float(argv[i + 1])
             i = i + 1
-            yres = float(argv[i+1])
+            yres = float(argv[i + 1])
             i = i + 1
-        elif argv[i] == '-a_nodata' and i < len(argv)-1:
-            nodata = float(argv[i+1])
+        elif argv[i] == '-a_nodata' and i < len(argv) - 1:
+            nodata = float(argv[i + 1])
             i = i + 1
-        elif argv[i] == '-scale' and i < len(argv)-1:
-            scale = float(argv[i+1])
+        elif argv[i] == '-scale' and i < len(argv) - 1:
+            scale = float(argv[i + 1])
             i = i + 1
-        elif argv[i] == '-offset' and i < len(argv)-1:
-            offset = float(argv[i+1])
+        elif argv[i] == '-offset' and i < len(argv) - 1:
+            offset = float(argv[i + 1])
             i = i + 1
-        elif argv[i] == '-mo' and i < len(argv)-1:
-            molist.append(argv[i+1])
+        elif argv[i] == '-mo' and i < len(argv) - 1:
+            molist.append(argv[i + 1])
             i = i + 1
         elif argv[i] == '-gcp' and i + 4 < len(argv):
-            pixel = float(argv[i+1])
+            pixel = float(argv[i + 1])
             i = i + 1
-            line = float(argv[i+1])
+            line = float(argv[i + 1])
             i = i + 1
-            x = float(argv[i+1])
+            x = float(argv[i + 1])
             i = i + 1
-            y = float(argv[i+1])
+            y = float(argv[i + 1])
             i = i + 1
-            if i + 1 < len(argv) and ArgIsNumeric(argv[i+1]):
-                z = float(argv[i+1])
+            if i + 1 < len(argv) and ArgIsNumeric(argv[i + 1]):
+                z = float(argv[i + 1])
                 i = i + 1
             else:
                 z = 0
-            gcp = gdal.GCP(x,y,z,pixel,line)
+            gcp = gdal.GCP(x, y, z, pixel, line)
             gcp_list.append(gcp)
-        elif argv[i] == '-unsetgt' :
+        elif argv[i] == '-unsetgt':
             unsetgt = True
         elif argv[i] == '-unsetstats':
             unsetstats = True
@@ -147,8 +151,29 @@ def gdal_edit(argv):
             unsetmd = True
         elif argv[i] == '-unsetnodata':
             unsetnodata = True
-        elif argv[i] == '-oo' and i < len(argv)-1:
-            open_options.append(argv[i+1])
+        elif argv[i] == '-oo' and i < len(argv) - 1:
+            open_options.append(argv[i + 1])
+            i = i + 1
+        elif argv[i].startswith('-colorinterp_')and i < len(argv) - 1:
+            band = int(argv[i][len('-colorinterp_'):])
+            val = argv[i + 1]
+            if val.lower() == 'red':
+                val = gdal.GCI_RedBand
+            elif val.lower() == 'green':
+                val = gdal.GCI_GreenBand
+            elif val.lower() == 'blue':
+                val = gdal.GCI_BlueBand
+            elif val.lower() == 'alpha':
+                val = gdal.GCI_AlphaBand
+            elif val.lower() == 'gray' or val.lower() == 'grey':
+                val = gdal.GCI_GrayIndex
+            elif val.lower() == 'undefined':
+                val = gdal.GCI_Undefined
+            else:
+                sys.stderr.write('Unsupported color interpretation %s.\n' % val +
+                                 'Only red, green, blue, alpha, gray, undefined are supported.\n')
+                return Usage()
+            colorinterp[band] = val
             i = i + 1
         elif argv[i][0] == '-':
             sys.stderr.write('Unrecognized option : %s\n' % argv[i])
@@ -164,11 +189,11 @@ def gdal_edit(argv):
     if datasetname is None:
         return Usage()
 
-    if (srs is None and lry is None and yres is None and not unsetgt
-            and not unsetstats and not stats and nodata is None
-            and len(molist) == 0 and not unsetmd and len(gcp_list) == 0
-            and not unsetnodata
-            and scale is None and offset is None):
+    if (srs is None and lry is None and yres is None and not unsetgt and
+            not unsetstats and not stats and nodata is None and
+            len(molist) == 0 and not unsetmd and len(gcp_list) == 0 and
+            not unsetnodata and len(colorinterp) == 0 and
+            scale is None and offset is None):
         print('No option specified')
         print('')
         return Usage()
@@ -197,9 +222,9 @@ def gdal_edit(argv):
 
     if open_options is not None:
         if ro:
-            ds = gdal.OpenEx(datasetname, gdal.OF_RASTER, open_options = open_options)
+            ds = gdal.OpenEx(datasetname, gdal.OF_RASTER, open_options=open_options)
         else:
-            ds = gdal.OpenEx(datasetname, gdal.OF_RASTER | gdal.OF_UPDATE, open_options = open_options)
+            ds = gdal.OpenEx(datasetname, gdal.OF_RASTER | gdal.OF_UPDATE, open_options=open_options)
     # GDAL 1.X compat
     elif ro:
         ds = gdal.Open(datasetname)
@@ -221,20 +246,25 @@ def gdal_edit(argv):
             ds.SetProjection(wkt)
 
     if lry is not None:
-        gt = [ ulx, (lrx - ulx) / ds.RasterXSize, 0,
-               uly, 0, (lry - uly) / ds.RasterYSize ]
+        gt = [ulx, (lrx - ulx) / ds.RasterXSize, 0,
+              uly, 0, (lry - uly) / ds.RasterYSize]
         ds.SetGeoTransform(gt)
 
     if yres is not None:
         gt = ds.GetGeoTransform()
         # Doh ! why is gt a tuple and not an array...
-        gt = [ gt[j] for j in range(6) ]
+        gt = [gt[j] for j in range(6)]
         gt[1] = xres
         gt[5] = yres
         ds.SetGeoTransform(gt)
 
     if unsetgt:
-        ds.SetGeoTransform([0,1,0,0,0,1])
+        # For now only the GTiff drivers understands full-zero as a hint
+        # to unset the geotransform
+        if ds.GetDriver().ShortName == 'GTiff':
+            ds.SetGeoTransform([0, 0, 0, 0, 0, 0])
+        else:
+            ds.SetGeoTransform([0, 1, 0, 0, 0, 1])
 
     if len(gcp_list) > 0:
         if wkt is None:
@@ -245,29 +275,29 @@ def gdal_edit(argv):
 
     if nodata is not None:
         for i in range(ds.RasterCount):
-            ds.GetRasterBand(i+1).SetNoDataValue(nodata)
+            ds.GetRasterBand(i + 1).SetNoDataValue(nodata)
     elif unsetnodata:
         for i in range(ds.RasterCount):
-            ds.GetRasterBand(i+1).DeleteNoDataValue()
+            ds.GetRasterBand(i + 1).DeleteNoDataValue()
 
     if scale is not None:
         for i in range(ds.RasterCount):
-            ds.GetRasterBand(i+1).SetScale(scale)
+            ds.GetRasterBand(i + 1).SetScale(scale)
 
     if offset is not None:
-       for i in range(ds.RasterCount):
-           ds.GetRasterBand(i+1).SetOffset(offset)
- 
+        for i in range(ds.RasterCount):
+            ds.GetRasterBand(i + 1).SetOffset(offset)
+
     if unsetstats:
         for i in range(ds.RasterCount):
-            band = ds.GetRasterBand(i+1)
+            band = ds.GetRasterBand(i + 1)
             for key in band.GetMetadata().keys():
                 if key.startswith('STATISTICS_'):
                     band.SetMetadataItem(key, None)
 
     if stats:
         for i in range(ds.RasterCount):
-            ds.GetRasterBand(i+1).ComputeStatistics(approx_stats)
+            ds.GetRasterBand(i + 1).ComputeStatistics(approx_stats)
 
     if len(molist) != 0:
         if unsetmd:
@@ -277,17 +307,22 @@ def gdal_edit(argv):
         for moitem in molist:
             equal_pos = moitem.find('=')
             if equal_pos > 0:
-                md[moitem[0:equal_pos]] = moitem[equal_pos+1:]
+                md[moitem[0:equal_pos]] = moitem[equal_pos + 1:]
         ds.SetMetadata(md)
     elif unsetmd:
         ds.SetMetadata({})
 
+    for band in colorinterp:
+        ds.GetRasterBand(band).SetColorInterpretation(colorinterp[band])
+
     ds = band = None
 
     return 0
 
+
 def main():
     return gdal_edit(sys.argv)
 
+
 if __name__ == '__main__':
     sys.exit(gdal_edit(sys.argv))
