@@ -1,13 +1,13 @@
 #!/usr/local/bin/python2.7
-#******************************************************************************
-#  $Id: gdal_proximity.py 33790 2016-03-26 12:42:12Z goatbar $
+# ******************************************************************************
+#  $Id: gdal_proximity.py 1121758f74589f8850193a5baced7ab1335a4b39 2018-04-20 11:27:52 +0200 Even Rouault $
 #
 #  Name:     gdalproximity
 #  Project:  GDAL Python Interface
 #  Purpose:  Application for computing raster proximity maps.
 #  Author:   Frank Warmerdam, warmerdam@pobox.com
 #
-#******************************************************************************
+# ******************************************************************************
 #  Copyright (c) 2008, Frank Warmerdam
 #  Copyright (c) 2009-2011, Even Rouault <even dot rouault at mines-paris dot org>
 #
@@ -28,12 +28,14 @@
 #  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 #  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 #  DEALINGS IN THE SOFTWARE.
-#******************************************************************************
+# ******************************************************************************
 
+import os.path
 import sys
 
 from osgeo import gdal
 
+
 def Usage():
     print("""
 gdal_proximity.py srcfile dstfile [-srcband n] [-dstband n]
@@ -44,11 +46,61 @@ gdal_proximity.py srcfile dstfile [-srcband n] [-dstband n]
                   [-fixed-buf-val n] [-q] """)
     sys.exit(1)
 
+
+def DoesDriverHandleExtension(drv, ext):
+    exts = drv.GetMetadataItem(gdal.DMD_EXTENSIONS)
+    return exts is not None and exts.lower().find(ext.lower()) >= 0
+
+
+def GetExtension(filename):
+    ext = os.path.splitext(filename)[1]
+    if ext.startswith('.'):
+        ext = ext[1:]
+    return ext
+
+
+def GetOutputDriversFor(filename):
+    drv_list = []
+    ext = GetExtension(filename)
+    for i in range(gdal.GetDriverCount()):
+        drv = gdal.GetDriver(i)
+        if (drv.GetMetadataItem(gdal.DCAP_CREATE) is not None or
+            drv.GetMetadataItem(gdal.DCAP_CREATECOPY) is not None) and \
+           drv.GetMetadataItem(gdal.DCAP_RASTER) is not None:
+            if len(ext) > 0 and DoesDriverHandleExtension(drv, ext):
+                drv_list.append(drv.ShortName)
+            else:
+                prefix = drv.GetMetadataItem(gdal.DMD_CONNECTION_PREFIX)
+                if prefix is not None and filename.lower().startswith(prefix.lower()):
+                    drv_list.append(drv.ShortName)
+
+    # GMT is registered before netCDF for opening reasons, but we want
+    # netCDF to be used by default for output.
+    if ext.lower() == 'nc' and len(drv_list) == 0 and \
+       drv_list[0].upper() == 'GMT' and drv_list[1].upper() == 'NETCDF':
+        drv_list = ['NETCDF', 'GMT']
+
+    return drv_list
+
+
+def GetOutputDriverFor(filename):
+    drv_list = GetOutputDriversFor(filename)
+    if len(drv_list) == 0:
+        ext = GetExtension(filename)
+        if len(ext) == 0:
+            return 'GTiff'
+        else:
+            raise Exception("Cannot guess driver for %s" % filename)
+    elif len(drv_list) > 1:
+        print("Several drivers matching %s extension. Using %s" % (ext, drv_list[0]))
+    return drv_list[0]
+
 # =============================================================================
 #     Mainline
 # =============================================================================
 
-format = 'GTiff'
+
+format = None
 creation_options = []
 options = []
 src_filename = None
@@ -59,16 +111,16 @@ creation_type = 'Float32'
 quiet_flag = 0
 
 gdal.AllRegister()
-argv = gdal.GeneralCmdLineProcessor( sys.argv )
+argv = gdal.GeneralCmdLineProcessor(sys.argv)
 if argv is None:
-    sys.exit( 0 )
+    sys.exit(0)
 
 # Parse command line arguments.
 i = 1
 while i < len(argv):
     arg = argv[i]
 
-    if arg == '-of':
+    if arg == '-of' or arg == '-f':
         i = i + 1
         format = argv[i]
 
@@ -82,27 +134,27 @@ while i < len(argv):
 
     elif arg == '-maxdist':
         i = i + 1
-        options.append( 'MAXDIST=' + argv[i] )
+        options.append('MAXDIST=' + argv[i])
 
     elif arg == '-values':
         i = i + 1
-        options.append( 'VALUES=' + argv[i] )
+        options.append('VALUES=' + argv[i])
 
     elif arg == '-distunits':
         i = i + 1
-        options.append( 'DISTUNITS=' + argv[i] )
+        options.append('DISTUNITS=' + argv[i])
 
     elif arg == '-nodata':
         i = i + 1
-        options.append( 'NODATA=' + argv[i] )
+        options.append('NODATA=' + argv[i])
 
     elif arg == '-use_input_nodata':
         i = i + 1
-        options.append( 'USE_INPUT_NODATA=' + argv[i] )
+        options.append('USE_INPUT_NODATA=' + argv[i])
 
     elif arg == '-fixed-buf-val':
         i = i + 1
-        options.append( 'FIXED_BUF_VAL=' + argv[i] )
+        options.append('FIXED_BUF_VAL=' + argv[i])
 
     elif arg == '-srcband':
         i = i + 1
@@ -133,7 +185,7 @@ if src_filename is None or dst_filename is None:
 #    Open source file
 # =============================================================================
 
-src_ds = gdal.Open( src_filename )
+src_ds = gdal.Open(src_filename)
 
 if src_ds is None:
     print('Unable to open %s' % src_filename)
@@ -146,9 +198,9 @@ srcband = src_ds.GetRasterBand(src_band_n)
 # =============================================================================
 
 try:
-    driver = gdal.IdentifyDriver( dst_filename )
+    driver = gdal.IdentifyDriver(dst_filename)
     if driver is not None:
-        dst_ds = gdal.Open( dst_filename, gdal.GA_Update )
+        dst_ds = gdal.Open(dst_filename, gdal.GA_Update)
         dstband = dst_ds.GetRasterBand(dst_band_n)
     else:
         dst_ds = None
@@ -159,13 +211,16 @@ except:
 #     Create output file.
 # =============================================================================
 if dst_ds is None:
-    drv = gdal.GetDriverByName(format)
-    dst_ds = drv.Create( dst_filename,
-                         src_ds.RasterXSize, src_ds.RasterYSize, 1,
-                         gdal.GetDataTypeByName(creation_type), creation_options )
+    if format is None:
+        format = GetOutputDriverFor(dst_filename)
 
-    dst_ds.SetGeoTransform( src_ds.GetGeoTransform() )
-    dst_ds.SetProjection( src_ds.GetProjectionRef() )
+    drv = gdal.GetDriverByName(format)
+    dst_ds = drv.Create(dst_filename,
+                        src_ds.RasterXSize, src_ds.RasterYSize, 1,
+                        gdal.GetDataTypeByName(creation_type), creation_options)
+
+    dst_ds.SetGeoTransform(src_ds.GetGeoTransform())
+    dst_ds.SetProjection(src_ds.GetProjectionRef())
 
     dstband = dst_ds.GetRasterBand(1)
 
@@ -176,10 +231,10 @@ if dst_ds is None:
 if quiet_flag:
     prog_func = None
 else:
-    prog_func = gdal.TermProgress
+    prog_func = gdal.TermProgress_nocb
 
-gdal.ComputeProximity( srcband, dstband, options,
-                       callback = prog_func )
+gdal.ComputeProximity(srcband, dstband, options,
+                      callback=prog_func)
 
 srcband = None
 dstband = None
